/* extension.js
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

/* exported init */

const { Clutter, Gio, GnomeBluetooth, GObject, St } = imports.gi;

const Gettext = imports.gettext.domain('removable-drive-status');
const _ = Gettext.gettext;

const ExtensionUtils = imports.misc.extensionUtils;
const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;
const ShellMountOperation = imports.ui.shellMountOperation;

var MountMenuItem = GObject.registerClass(
    class MountMenuItem extends PopupMenu.PopupBaseMenuItem {
        _init(mount) {
            super._init();

            let icon_name = 'drive-harddisk-usb-symbolic';
            if (mount.get_symbolic_icon() != null && 
                    mount.get_symbolic_icon().get_names() != null && 
                    mount.get_symbolic_icon().get_names().length > 0) {
                icon_name = mount.get_symbolic_icon().get_names()[0];
            }
            let mount_icon = new St.Icon({
                icon_name: icon_name,
                style_class: 'popup-menu-icon',
            });
            this.add_child(mount_icon);

            this.label = new St.Label({ text: mount.get_name(), x_align: Clutter.ActorAlign.START, x_expand: true });
            this.add_child(this.label);
            this.label_actor = this.label;
    
            this.mount = mount;
    
            this.connect('destroy', this._onDestroy.bind(this));
    
            let ejectIcon = new St.Icon({
                icon_name: 'media-eject-symbolic',
                style_class: 'popup-menu-icon',
            });
            let ejectButton = new St.Button({ child: ejectIcon, x_align: Clutter.ActorAlign.END });
            ejectButton.connect('clicked', this._eject.bind(this));
            this.add_child(ejectButton);
    
            this._changedId = mount.connect('changed', this._syncVisibility.bind(this));
            this._syncVisibility();
        }
    
        _onDestroy() {
            if (this._changedId) {
                this.mount.disconnect(this._changedId);
                this._changedId = 0;
            }
    
            super.destroy();
        }
    
        _isInteresting() {
            if (!this.mount.can_eject() && !this.mount.can_unmount())
                return false;
            if (this.mount.is_shadowed())
                return false;
    
            let volume = this.mount.get_volume();
    
            if (!volume) {
                // probably a GDaemonMount, could be network or
                // local, but we can't tell; assume it's local for now
                return true;
            }
    
            return volume.get_identifier('class') !== 'network';
        }
    
        _syncVisibility() {
            this.visible = this._isInteresting();
        }
    
        _eject() {
            let unmountArgs = [
                Gio.MountUnmountFlags.NONE,
                new ShellMountOperation.ShellMountOperation(this.mount).mountOp,
                null, // Gio.Cancellable
            ];
    
            if (this.mount.can_eject()) {
                this.mount.eject_with_operation(...unmountArgs,
                    this._ejectFinish.bind(this));
            } else {
                this.mount.unmount_with_operation(...unmountArgs,
                    this._unmountFinish.bind(this));
            }
        }
    
        _unmountFinish(mount, result) {
            try {
                mount.unmount_with_operation_finish(result);
            } catch (e) {
                this._reportFailure(e);
            }
        }
    
        _ejectFinish(mount, result) {
            try {
                mount.eject_with_operation_finish(result);
            } catch (e) {
                this._reportFailure(e);
            }
        }
    
        _reportFailure(exception) {
            // TRANSLATORS: %s is the filesystem name
            let msg = _('Ejecting drive “%s” failed:').format(this.mount.get_name());
            Main.notifyError(msg, exception.message);
        }
    
        activate(event) {
            let uri = this.mount.get_root().get_uri();
            let context = global.create_app_launch_context(event.get_time(), -1);
            Gio.AppInfo.launch_default_for_uri(uri, context);
    
            super.activate(event);
        }
    });

var Indicator = GObject.registerClass(
class Indicator extends PanelMenu.SystemIndicator {
    _init() {
        super._init();

        this._indicator = this._addIndicator();
        this._indicator.icon_name = 'media-removable-symbolic';

        this._item = new PopupMenu.PopupSubMenuMenuItem(_("Removable drives"), true);
        this._item.icon.icon_name = this._indicator.icon_name;
        this.menu.addMenuItem(this._item);

        this._mounts = [];
        this._monitor = Gio.VolumeMonitor.get();
        this._addedId = this._monitor.connect('mount-added', (monitor, mount) => {
            this._addMount(mount);
            this._updateMenuVisibility();
        });
        this._removedId = this._monitor.connect('mount-removed', (monitor, mount) => {
            this._removeMount(mount);
            this._updateMenuVisibility();
        });

        this._monitor.get_mounts().forEach(this._addMount.bind(this));
        this._updateMenuVisibility();
    }

    _addMount(mount) {
        let item = new MountMenuItem(mount);
        this._mounts.unshift(item);
        this._item.menu.addMenuItem(item, 0);
    }

    _removeMount(mount) {
        for (let i = 0; i < this._mounts.length; i++) {
            let item = this._mounts[i];
            if (item.mount === mount) {
                item.destroy();
                this._mounts.splice(i, 1);
                return;
            }
        }
        log('Removing a mount that was never added to the menu');
    }

    _updateMenuVisibility() {
        if (this._mounts.filter(i => i.visible).length > 0) {
            this.show();
            this._item.show();
        } else {
            this.hide();
            this._item.hide();
        }
    }

    _onDestroy() {
        if (this._addedId) {
            this._monitor.disconnect(this._addedId);
            this._monitor.disconnect(this._removedId);
            this._addedId = 0;
            this._removedId = 0;
        }

        super._onDestroy();
    }
}
);

class Extension {
    constructor() {
        this.aggregateMenu = Main.panel.statusArea['aggregateMenu'];
    }

    enable() {
        this.indicator = new Indicator();
        this.aggregateMenu._indicators.insert_child_at_index(this.indicator, 0);

        let items = this.aggregateMenu.menu._getMenuItems(), position = 0;
        for (let i = 0; i < items.length; i++) {
            if (items[i] instanceof PopupMenu.PopupSeparatorMenuItem) {
                position = i;
            }
        }

        if (position > 0) {
            position -= 1;
        }

        this.aggregateMenu.menu.addMenuItem(this.indicator.menu, position);
    }

    disable() {
        this.aggregateMenu._indicators.remove_child(this.indicator);
        this.indicator.menu.destroy();
        this.indicator.destroy()
    }
}

function init() {
    ExtensionUtils.initTranslations();
    return new Extension();
}
